@@ -2,7 +2,7 @@
  Project: Graphos
  GRBezierPath.m
 
- Copyright (C) 2000-2013 GNUstep Application Project
+ Copyright (C) 2000-2015 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -31,6 +31,10 @@ static double k = 0.025;
 
 @implementation GRBezierPath
 
+- (GRObjectEditor *)allocEditor
+{
+  return [[GRBezierPathEditor alloc] initEditor:self];
+}
 
 /** initializes by using the properties array as defaults */
 - (id)initInView:(GRDocView *)aView
@@ -41,7 +45,6 @@ static double k = 0.025;
   if(self)
     {
       controlPoints = [[NSMutableArray alloc] initWithCapacity: 1];
-      editor = [[GRBezierPathEditor alloc] initEditor:self];
     }
 
   return self;
@@ -51,28 +54,57 @@ static double k = 0.025;
         inView:(GRDocView *)aView
         zoomFactor:(CGFloat)zf
 {
-  NSArray *psops, *linearr;
-  NSString *str;
-  NSPoint p, pp[3];
-  GRBezierControlPoint *prevcp;
-  double distx, disty;
-  NSUInteger i, count;
-
   self = [super init];
   if(self != nil)
     {
+      NSArray *psops, *linearr;
+      NSString *str;
+      NSPoint p, pp[3];
+      GRBezierControlPoint *prevcp;
+      double distx, disty;
+      NSUInteger i, count;
+      NSArray *points;
       CGFloat strokeCol[4];
       CGFloat fillCol[4];
       CGFloat strokeAlpha;
       CGFloat fillAlpha;
       id obj;
+      BOOL symm;
       
-      editor = [[GRBezierPathEditor alloc] initEditor:self];
+      psops = nil;
+      linearr = nil;
       docView = aView;
       zmFactor = zf;
+      editor = [self allocEditor];
       myPath = [[NSBezierPath bezierPath] retain];
       [myPath setCachesBezierPath: NO];
       controlPoints = [[NSMutableArray alloc] initWithCapacity: 1];
+      points = [description objectForKey: @"points"];
+      for (i = 0; i < [points count]; i++)
+        {
+          GRBezierHandle h;
+          GRBezierControlPoint *cp;
+
+          linearr = [[points objectAtIndex: i] componentsSeparatedByString: @" "];
+          h.firstHandle.x = [[linearr objectAtIndex: 0] floatValue];
+          h.firstHandle.y = [[linearr objectAtIndex: 1] floatValue];
+          h.firstHandleRect = NSMakeRect(h.firstHandle.x-2, h.firstHandle.y-2, 4, 4);
+          h.center.x = [[linearr objectAtIndex: 2] floatValue];
+          h.center.y = [[linearr objectAtIndex: 3] floatValue];
+          h.centerRect = NSMakeRect(h.center.x-3, h.center.y-3, 6, 6);
+          h.secondHandle.x = [[linearr objectAtIndex: 4] floatValue];
+          h.secondHandle.y = [[linearr objectAtIndex: 5] floatValue];
+          h.secondHandleRect = NSMakeRect(h.secondHandle.x-2, h.secondHandle.y-2, 4, 4);
+          symm = (BOOL)[[linearr objectAtIndex: 6] intValue];
+
+          cp = [[GRBezierControlPoint alloc] initAtPoint:h.center forPath:self zoomFactor:zmFactor];
+          [cp setBezierHandle:h];
+          [cp setSymmetricalHandles:symm];
+          [controlPoints addObject:cp];
+          [cp release];
+        }
+      [self confirmNewCurve];
+      [self remakePath];
       psops = [description objectForKey: @"psdata"];
       for(i = 0; i < [psops count]; i++)
         {
@@ -208,10 +240,8 @@ static double k = 0.025;
 - (NSDictionary *)objectDescription
 {
   NSMutableDictionary *dict;
-  NSMutableArray *psops;
+  NSMutableArray *points;
   NSString *str;
-  NSBezierPathElement type;
-  NSPoint p[3];
   NSUInteger i;
   CGFloat strokeCol[3];
   CGFloat fillCol[3];
@@ -256,21 +286,22 @@ static double k = 0.025;
   [dict setObject:[NSNumber numberWithBool:visible] forKey: @"visible"];
   [dict setObject:[NSNumber numberWithBool:locked] forKey: @"locked"];
 
-  psops = [NSMutableArray arrayWithCapacity: 1];
-  // FIXME/TODO in a new file release, this list should be generated from the control-points instead */
-  for(i = 0; i < [myPath elementCount]; i++)
+  points = [NSMutableArray arrayWithCapacity: 1];
+  for (i = 0; i < [controlPoints count]; i++)
     {
-      type = [myPath elementAtIndex: i associatedPoints: p];
-      if(type == NSMoveToBezierPathElement)
-	str = [NSString stringWithFormat: @"%.3f %.3f moveto", p[0].x / zmFactor, p[0].y / zmFactor];
-      else if(type == NSLineToBezierPathElement)
-	str = [NSString stringWithFormat: @"%.3f %.3f lineto", p[0].x / zmFactor, p[0].y / zmFactor];
-      else if(type == NSCurveToBezierPathElement)
-	str = [NSString stringWithFormat: @"%.3f %.3f %.3f %.3f %.3f %.3f curveto",
-			p[0].x / zmFactor, p[0].y / zmFactor, p[1].x / zmFactor, p[1].y / zmFactor, p[2].x / zmFactor, p[2].y / zmFactor];
-        [psops addObject: str];
+      GRBezierControlPoint *cp;
+      GRBezierHandle handle;
+
+      cp = [controlPoints objectAtIndex:i];
+      handle = [cp bzHandle];
+      str = [NSString stringWithFormat: @"%f %f %f %f %f %f %d",
+                      handle.firstHandle.x, handle.firstHandle.y,
+                      handle.center.x, handle.center.y,
+                      handle.secondHandle.x, handle.secondHandle.y,
+                      [cp symmetricalHandles]];
+      [points addObject:str];
     }
-  [dict setObject: psops forKey: @"psdata"];
+  [dict setObject: points forKey:@ "points"];
   
   return dict;
 }
@@ -288,11 +319,15 @@ static double k = 0.025;
   e = [controlPoints objectEnumerator];
   while ((cp = [e nextObject]))
     {
-      [cpsCopy addObject:[cp copy]];
+      GRBezierControlPoint *cpCopy;
+
+      cpCopy = [cp copy];
+      [cpsCopy addObject: cpCopy];
+      [cpCopy release];
     }
   
   objCopy->controlPoints = cpsCopy;
-  
+  objCopy->calculatingHandles = calculatingHandles;
   return objCopy;
 }
 
@@ -523,41 +558,66 @@ static double k = 0.025;
 
 - (void)confirmNewCurve
 {
-    calculatingHandles = NO;
-    if([controlPoints count] == 1)
-        return;
-    if([self isPoint: (GRBezierControlPoint *)currentPoint onPoint: [controlPoints objectAtIndex: 0]])
-        [(GRBezierPathEditor *)editor setIsDone:YES];
+  if (!controlPoints || [controlPoints count] == 0)
+    return;
+  calculatingHandles = NO;
+  if([controlPoints count] == 1)
+    return;
+  if([self isPoint: (GRBezierControlPoint *)currentPoint onPoint: [controlPoints objectAtIndex: 0]])
+    [(GRBezierPathEditor *)editor setIsDone:YES];
+  
+  [self remakePath];
 }
 
 - (void)remakePath
 {
   GRBezierControlPoint *cp, *prevcp, *mtopoint;
-  GRBezierHandle handle1, handle2;
   NSInteger i;
 
   [myPath removeAllPoints];
+  if (!controlPoints || [controlPoints count] == 0)
+    return;
+
   mtopoint = [controlPoints objectAtIndex: 0];
   [myPath moveToPoint: GRpointZoom([mtopoint center], zmFactor)];
   for(i = 1; i < [controlPoints count]; i++)
     {
+      GRBezierHandle handle1, handle2;
+      BOOL isLine;
+
       cp = [controlPoints objectAtIndex: i];
       prevcp = [controlPoints objectAtIndex: i -1];
-      if([prevcp isActiveHandle] || [cp isActiveHandle])
-	{
-	  handle1 = [prevcp bzHandle];
-	  handle2 = [cp bzHandle];
-	  [myPath curveToPoint: GRpointZoom([cp center], zmFactor)
-		  controlPoint1: GRpointZoom(handle1.firstHandle, zmFactor)
-		  controlPoint2: GRpointZoom(handle2.secondHandle, zmFactor)];
-	}
+      handle1 = [prevcp bzHandle];
+      handle2 = [cp bzHandle];
+
+      /* we have a line if the start and end control points have respectively
+         right and left center-coincident handles */
+      isLine = NO;
+      if (NSEqualPoints(handle1.center, handle1.secondHandle) && NSEqualPoints(handle2.center, handle2.firstHandle))
+        isLine = YES;
+      
+      if (isLine)
+        {
+          [myPath lineToPoint: GRpointZoom([cp center], zmFactor)];
+        }
       else
-	{
-	  [myPath lineToPoint: GRpointZoom([cp center], zmFactor)];
-	}
+        {
+          [myPath curveToPoint: GRpointZoom([cp center], zmFactor)
+                 controlPoint1: GRpointZoom(handle1.firstHandle, zmFactor)
+                 controlPoint2: GRpointZoom(handle2.secondHandle, zmFactor)];
+          [cp setPointPosition:GRPointMiddle];
+        }
+
       if([self isPoint: cp onPoint: mtopoint])
 	[(GRBezierPathEditor *)editor setIsDone:YES];
     }
+
+  /* if the path is open, set the Start ad End points controls */
+  if (!NSEqualPoints([(GRBezierControlPoint *)[controlPoints objectAtIndex:0] center], [(GRBezierControlPoint *)[controlPoints objectAtIndex:[controlPoints count]-1] center]))
+    {
+      [[controlPoints objectAtIndex:0] setPointPosition:GRPointStart];
+      [[controlPoints objectAtIndex:[controlPoints count]-1] setPointPosition:GRPointEnd];
+    }
 }
 
 
@@ -732,23 +792,6 @@ static double k = 0.025;
 }
 
 
-
-
-- (void)unselectOtherControls:(GRBezierControlPoint *)cp
-{
-  GRBezierControlPoint *ctrlp;
-  NSUInteger i;
-
-  currentPoint = cp;
-  for(i = 0; i < [controlPoints count]; i++)
-    {
-      ctrlp = [controlPoints objectAtIndex: i];
-      if(ctrlp != cp)
-        [ctrlp unselect];
-    }
-}
-
-
 - (void)draw
 {
   GRBezierControlPoint *cp;
