@@ -2,7 +2,7 @@
  Project: Graphos
  GRDocView.m
 
- Copyright (C) 2000-2013 GNUstep Application Project
+ Copyright (C) 2000-2017 GNUstep Application Project
 
  Author: Enrico Sersale (original GDraw implementation)
  Author: Ing. Riccardo Mottola
@@ -24,6 +24,7 @@
 
 #import "GRDocView.h"
 #import "Graphos.h"
+#import "GRDrawableObject.h"
 #import "GRFunctions.h"
 #import "GRBezierPath.h"
 #import "GRBox.h"
@@ -33,13 +34,17 @@
 #import "GRCircle.h"
 #import "GRCircleEditor.h"
 #import "GRPropsEditor.h"
+#import "GRImage.h"
 
 #define UNDO_ACTION_OBJPROPS @"Change Object Properties"
+#define UNDO_ACTION_CP_SYMMETRIC @"Change Point to Symmetric"
+#define UNDO_ACTION_CP_CUSP @"Change Point to Cusp"
 
 #define ZOOM_FACTORS 13
 #define STD_ZOOM_INDEX 5
 float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.5, 3, 4, 6};
 
+
 @implementation GRDocView
 
 
@@ -89,47 +94,56 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   NSMutableArray *objectOrder;
   NSString *str = nil;
   id obj;
-  NSUInteger i;
+  NSUInteger counter;
   NSUInteger p = 0;
   NSUInteger c = 0;
   NSUInteger t = 0;
   NSUInteger b = 0;
+  NSUInteger i = 0;
   
   objsdict = [NSMutableDictionary dictionaryWithCapacity: 1];
   objectOrder = [NSMutableArray arrayWithCapacity: [objects count]];
-  for(i = 0; i < [objects count]; i++)
+  for(counter = 0; counter < [objects count]; counter++)
     {
-      obj = [objects objectAtIndex: i];
+      obj = [objects objectAtIndex: counter];
       NSLog(@"class: %@", [obj className]);
       if([obj isKindOfClass: [GRBezierPath class]])
         {
-	  str = [NSString stringWithFormat: @"path%i", p];
+	  str = [NSString stringWithFormat: @"path%lu", (unsigned long)p];
 	  p++;
         }
+      else if([obj isKindOfClass: [GRImage class]])
+        {
+	  str = [NSString stringWithFormat: @"image%lu", (unsigned long)i];
+	  i++;
+        }
       else if([obj isKindOfClass: [GRBox class]])
         {
-	  str = [NSString stringWithFormat: @"box%i", b];
+	  str = [NSString stringWithFormat: @"box%lu", (unsigned long)b];
 	  b++;
         }
       else if([obj isKindOfClass: [GRCircle class]])
         {
-	  str = [NSString stringWithFormat: @"circle%i", c];
+	  str = [NSString stringWithFormat: @"circle%lu", (unsigned long)c];
 	  c++;
         }
       else if([obj isKindOfClass: [GRText class]])
         {
-	  str = [NSString stringWithFormat: @"text%i", t];
+	  str = [NSString stringWithFormat: @"text%lu", (unsigned long)t];
 	  t++;
         }
       else
 	{
 	  [NSException raise:@"Unhandled object type" format:@"%@", [obj class]];
 	}
-      [objectOrder addObject: str];
-      [objsdict setObject: [obj objectDescription] forKey: str];
+      if (str)
+        {
+          [objectOrder addObject: str];
+          [objsdict setObject: [obj objectDescription] forKey: str];
+        }
     }
-  [objsdict setValue:[NSNumber numberWithFloat:FILE_FORMAT_VERSION] forKey:@"Version"];
-  [objsdict setValue:objectOrder forKey:@"Order"];
+  [objsdict setObject:[NSNumber numberWithFloat:FILE_FORMAT_VERSION] forKey:@"Version"];
+  [objsdict setObject:objectOrder forKey:@"Order"];
   return [NSDictionary dictionaryWithDictionary: objsdict];
 }
 
@@ -143,13 +157,15 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
     GRText *gGRText;
     GRBox *box;
     GRCircle *circle;
-    int i;
+    GRImage *image;
+    NSUInteger i;
     float version;
     NSNumber *versionNumber;
 
     if(!dict)
       return NO;
 
+    [objects removeAllObjects];
     version = 0.0;
     versionNumber = [dict objectForKey:@"Version"];
     if (versionNumber)
@@ -195,6 +211,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
 	      {
 		circle = [[GRCircle alloc] initFromData: objdict
 						 inView: self zoomFactor: zFactor];
+                [circle setCircle:YES];
 		[objects addObject: circle];
 		[circle release];
 	      }
@@ -244,10 +261,20 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
 		[objects addObject: box];
 		[box release];
 	      }
+	    else if([key rangeOfString: @"image"].length)
+	      {
+		image = [[GRImage alloc] initFromData: objdict
+					   inView: self zoomFactor: zFactor];
+		[objects addObject: image];
+		[image release];
+	      }
 	    else if([key rangeOfString: @"circle"].length)
 	      {
 		circle = [[GRCircle alloc] initFromData: objdict
 						 inView: self zoomFactor: zFactor];
+                /* 0.5 and prior only had circles, not ovals */
+                if (version < 0.6)
+                  [circle setCircle:YES];
 		[objects addObject: circle];
 		[circle release];
 	      }
@@ -257,6 +284,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
 	      }
 	  }
       }
+    [self setNeedsDisplay:YES];
     return YES;
 }
 
@@ -265,7 +293,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   GRBezierPath *path;
   GRPropsEditor *objInspector;
 
-  objInspector = [[[NSApplication sharedApplication] delegate] objectInspector];
+  objInspector = [(Graphos *)[[NSApplication sharedApplication] delegate] objectInspector];
 
   path = [[GRBezierPath alloc] initInView: self
 			       zoomFactor: zFactor
@@ -280,11 +308,11 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
 - (void)addTextAtPoint:(NSPoint)p
 {
   GRText *gdtxt;
-  int i;
+  NSInteger i;
   NSUndoManager *uMgr;
   GRPropsEditor *objInspector;
 
-  objInspector = [[[NSApplication sharedApplication] delegate] objectInspector];
+  objInspector = [(Graphos *)[[NSApplication sharedApplication] delegate] objectInspector];
     
   uMgr = [self undoManager];
   /* save the method on the undo stack */
@@ -293,7 +321,6 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
     
   [self saveCurrentObjects];
     
-  NSLog(@"AddTextAtPoint");
   for(i = 0; i < [objects count]; i++)
     [[[objects objectAtIndex: i] editor] unselect];
 
@@ -315,7 +342,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   GRBox *box;
   GRPropsEditor *objInspector;
 
-  objInspector = [[[NSApplication sharedApplication] delegate] objectInspector];
+  objInspector = [(Graphos *)[[NSApplication sharedApplication] delegate] objectInspector];
 
   box = [[GRBox alloc] initInView: self
 		       zoomFactor: zFactor
@@ -334,7 +361,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   GRCircle *circle;
   GRPropsEditor *objInspector;
 
-  objInspector = [[[NSApplication sharedApplication] delegate] objectInspector];
+  objInspector = [(Graphos *)[[NSApplication sharedApplication] delegate] objectInspector];
 
   circle = [[GRCircle alloc] initInView: self
 			     zoomFactor: zFactor
@@ -352,7 +379,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
 {
   id obj, duplObj;
   NSMutableArray *duplObjs;
-  int i;
+  NSUInteger i;
 
   duplObjs = [NSMutableArray arrayWithCapacity: 1];
 
@@ -407,7 +434,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
 {
     id obj;
     NSMutableArray *deleted;
-    int i, count;
+    NSUInteger i, count;
 
     deleted = [NSMutableArray arrayWithCapacity: 1];
 
@@ -731,7 +758,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
   [uMgr setActionName:@"Edit Path"];
   
-  [self saveCurrentObjects];
+  [self saveCurrentObjectsDeep];
 
   /* we look for a path that is selected and process editing
      we no longer auto-select the object for editing though, nor automatically unselect the editing of the object */
@@ -775,15 +802,15 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   NSUInteger i;
   NSUndoManager *uMgr;
     
-    uMgr = [self undoManager];
-    /* save the method on the undo stack */
-    [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
-    [uMgr setActionName:@"Edit Text"];
+  uMgr = [self undoManager];
+  /* save the method on the undo stack */
+  [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+  [uMgr setActionName:@"Edit Text"];
     
-    [self saveCurrentObjectsDeep];
+  [self saveCurrentObjectsDeep];
 
-    for(i = 0; i < [objects count]; i++)
-        [[[objects objectAtIndex: i] editor] unselect];
+  for(i = 0; i < [objects count]; i++)
+    [[[objects objectAtIndex: i] editor] unselect];
 
     for(i = 0; i < [objects count]; i++)
     {
@@ -917,6 +944,88 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   return YES;
 }
 
+- (void)changePointsOfCurrentPathToSymmetric:(id)sender
+{
+  NSUndoManager *uMgr;
+  NSUInteger i;
+  NSArray *points;
+  GRBezierPath *path;
+
+  path = nil;
+  for(i = 0; i < [objects count]; i++)
+    {
+      GRDrawableObject *obj;
+      obj = [objects objectAtIndex: i];
+      
+      if([[obj editor] isSelected] && [obj isKindOfClass:[GRBezierPath class]])
+        {
+          path = (GRBezierPath *)obj;
+        }
+    }
+  
+  if (!path)
+    return;
+
+  points = [(GRBezierPathEditor *)[path editor] selectedControlPoints];
+  if (!points || [points count] == 0)
+    return;
+
+  uMgr = [self undoManager];
+  /* save the method on the undo stack, but stack actions */
+  if ([[uMgr undoActionName] isEqualToString: UNDO_ACTION_CP_SYMMETRIC] == NO)
+    {
+      [self saveCurrentObjectsDeep];
+      [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+      [uMgr setActionName: UNDO_ACTION_CP_SYMMETRIC];
+    }
+
+  for (i = 0; i < [points count]; i++)
+    {
+      [(GRBezierControlPoint *)[points objectAtIndex: i] setSymmetricalHandles:YES];
+    }
+}
+
+- (void)changePointsOfCurrentPathToCusp:(id)sender
+{
+  NSUndoManager *uMgr;
+  NSUInteger i;
+  NSArray *points;
+  GRBezierPath *path;
+
+  path = nil;
+  for(i = 0; i < [objects count]; i++)
+    {
+      GRDrawableObject *obj;
+      obj = [objects objectAtIndex: i];
+      
+      if([[obj editor] isSelected] && [obj isKindOfClass:[GRBezierPath class]])
+        {
+          path = (GRBezierPath *)obj;
+        }
+    }
+  
+  if (!path)
+    return;
+
+  points = [(GRBezierPathEditor *)[path editor] selectedControlPoints];
+  if (!points || [points count] == 0)
+    return;
+
+  uMgr = [self undoManager];
+  /* save the method on the undo stack, but stack actions */
+  if ([[uMgr undoActionName] isEqualToString: UNDO_ACTION_CP_CUSP] == NO)
+    {
+      [self saveCurrentObjectsDeep];
+      [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
+      [uMgr setActionName: UNDO_ACTION_CP_CUSP];
+    }
+
+  for (i = 0; i < [points count]; i++)
+    {
+      [(GRBezierControlPoint *)[points objectAtIndex: i] setSymmetricalHandles:NO];
+    }
+}
+
 - (void)subdividePathAtPoint:(NSPoint)p splitIt:(BOOL)split
 {
   id obj;
@@ -942,6 +1051,9 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   NSMutableDictionary *propDict;
   NSUInteger i;
   NSUInteger selectedObjects;
+  NSUInteger circleObjNum;
+  NSUInteger boxObjNum;
+  NSUInteger bezObjNum;
   NSUInteger pathObjNum;
   NSUInteger textObjNum;
   NSNumber *num;
@@ -950,6 +1062,9 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
     return nil;
 
   selectedObjects = 0;
+  circleObjNum = 0;
+  boxObjNum = 0;
+  bezObjNum = 0;
   pathObjNum = 0;
   textObjNum = 0;
   propDict = [NSMutableDictionary dictionaryWithCapacity: 1];
@@ -982,6 +1097,20 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
               [propDict setObject: num forKey: @"miterlimit"];
               num = [NSNumber numberWithFloat: [obj lineWidth]];
               [propDict setObject: num forKey: @"linewidth"];
+              if([obj isKindOfClass: [GRCircle class]])
+                {
+                  circleObjNum++;
+                  num = [NSNumber numberWithBool: [obj circle]];
+                  [propDict setObject: num forKey: @"circle"];
+                }
+              else if([obj isKindOfClass: [GRBox class]])
+                {
+                  boxObjNum++;
+                }
+              else if([obj isKindOfClass: [GRBezierPath class]])
+                {
+                  bezObjNum++;
+                }
             }
           else if([obj isKindOfClass: [GRText class]])
             {
@@ -1030,6 +1159,8 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   
   for(i = 0; i < [objects count]; i++)
     {
+      NSNumber *num;
+
       obj = [objects objectAtIndex: i];
       if([[obj editor] isSelected])
         {
@@ -1037,18 +1168,41 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
       
           if([obj isKindOfClass: [GRBezierPath class]] || [obj isKindOfClass: [GRBox class]] || [obj isKindOfClass: [GRCircle class]])
             {
-              [obj setFlat: [[properties objectForKey: @"flatness"] floatValue]];
-              [obj setLineJoin: [[properties objectForKey: @"linejoin"] intValue]];
-              [obj setLineCap: [[properties objectForKey: @"linecap"] intValue]];
-              [obj setMiterLimit: [[properties objectForKey: @"miterlimit"] floatValue]];
-              [obj setLineWidth: [[properties objectForKey: @"linewidth"] floatValue]];
+              num = [properties objectForKey: @"flatness"];
+              if (num)
+                [obj setFlat: [num floatValue]];
+
+              num = [properties objectForKey: @"linejoin"];
+              if (num)
+                [obj setLineJoin: [num intValue]];
+
+              num = [properties objectForKey: @"linecap"];
+              if (num)
+                [obj setLineCap: [num intValue]];
+
+              num = [properties objectForKey: @"miterlimit"];
+              if (num)
+                [obj setMiterLimit: [num floatValue]];
+
+              num = [properties objectForKey: @"linewidth"];
+              if (num)
+                [obj setLineWidth: [num floatValue]];
             }
-          [obj setStroked: [[properties objectForKey: @"stroked"] boolValue]];
+          num = [properties objectForKey: @"stroked"];
+          if (num)
+            [obj setStroked: [num boolValue]];
+
           newColor = (NSColor *)[properties objectForKey: @"strokecolor"];
-          [obj setStrokeColor: newColor];
-          [obj setFilled: [[properties objectForKey: @"filled"] boolValue]];
+          if (newColor)
+            [obj setStrokeColor: newColor];
+
+          num = [properties objectForKey: @"filled"];
+          if (num)
+            [obj setFilled: [num boolValue]];
+
           newColor = (NSColor *)[properties objectForKey: @"fillcolor"];
-          [obj setFillColor: newColor];
+          if (newColor)
+            [obj setFillColor: newColor];
         }
     }
   [self setNeedsDisplay: YES];
@@ -1369,16 +1523,17 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
 {
   NSMutableArray *types;
   NSPasteboard *pboard;
-  id obj;
+  GRDrawableObject *dObj;
   NSMutableArray *objsdesc;
   NSUInteger i;
 
+  dObj = nil;
   objsdesc = [NSMutableArray arrayWithCapacity: 1];
   for(i = 0; i < [objects count]; i++)
     {
-      obj = [objects objectAtIndex: i];
-      if([[obj editor] isGroupSelected])
-	[objsdesc addObject: [obj objectDescription]];
+      dObj = [objects objectAtIndex: i];
+      if([[dObj editor] isGroupSelected])
+	[objsdesc addObject: [dObj objectDescription]];
     }
   
   if([objsdesc count])
@@ -1387,13 +1542,24 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
       pboard = [NSPasteboard generalPasteboard];
       [pboard declareTypes: types owner: self];
       [pboard setString:[objsdesc description] forType: @"GRObjectPboardType"];
+
+      /* if we have just a single image selected, also add a standard TIFF image */
+      if ([objsdesc count] == 1 && [dObj isKindOfClass:[GRImage class]])
+        {
+          GRImage *grImg;
+
+          [types addObject:NSTIFFPboardType];
+          grImg = (GRImage *)dObj;
+          [pboard setData:[[grImg image] TIFFRepresentation] forType:NSTIFFPboardType];
+        }
     }
 }
 
 - (void)paste:(id)sender
 {
   NSPasteboard *pboard;
-  NSArray *types;
+  NSArray *grTypes;
+  NSString *imgType;
   NSArray *descriptions;
   NSDictionary *objdesc;
   id obj;
@@ -1401,6 +1567,7 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   NSUInteger i;
   NSUndoManager *uMgr;
   
+  // FIXME we should push the undo stack only if copy is certain to succeed
   uMgr = [self undoManager];
   /* save the method on the undo stack */
   [[uMgr prepareWithInvocationTarget: self] restoreLastObjects];
@@ -1409,38 +1576,74 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   [self saveCurrentObjects];
 
   pboard = [NSPasteboard generalPasteboard];
-  types = [NSArray arrayWithObject: @"GRObjectPboardType"];
-  if([[pboard availableTypeFromArray: types] isEqualToString: @"GRObjectPboardType"])
+  grTypes = [NSArray arrayWithObject: @"GRObjectPboardType"];
+  imgType = [pboard availableTypeFromArray:[NSArray arrayWithObjects:NSTIFFPboardType, nil]];
+  NSLog(@"%@", [pboard types]);
+  if([[pboard availableTypeFromArray: grTypes] isEqualToString: @"GRObjectPboardType"])
     {
-        descriptions = [[pboard stringForType: @"GRObjectPboardType"] propertyList];
+      descriptions = [[pboard stringForType: @"GRObjectPboardType"] propertyList];
 
-        for(i = 0; i < [descriptions count]; i++)
+      for(i = 0; i < [descriptions count]; i++)
         {
-            objdesc = [descriptions objectAtIndex: i];
-            str = [objdesc objectForKey: @"type"];
+          objdesc = [descriptions objectAtIndex: i];
+          str = [objdesc objectForKey: @"type"];
 
-            obj = nil;
-            if([str isEqualToString: @"path"])
-                obj = [GRBezierPath alloc];
-            else if([str isEqualToString: @"text"])
-                obj = [GRText alloc];
-            else if([str isEqualToString: @"box"])
-                obj = [GRBox alloc];
-            else if([str isEqualToString: @"circle"])
-                obj = [GRCircle alloc];
-            else
-                NSLog(@"Unknown object to paste");
-            if (obj != nil)
+          obj = nil;
+          if([str isEqualToString: @"path"])
+            obj = [GRBezierPath alloc];
+          else if([str isEqualToString: @"text"])
+            obj = [GRText alloc];
+          else if([str isEqualToString: @"box"])
+            obj = [GRBox alloc];
+          else if([str isEqualToString: @"circle"])
+            obj = [GRCircle alloc];
+          else if([str isEqualToString: @"image"])
+            obj = [GRImage alloc];
+          else
+            NSLog(@"Unknown object to paste");
+          if (obj != nil)
             {
-                obj = [obj initFromData: objdesc
-                                 inView: self zoomFactor: zFactor];
-                [objects addObject: obj];
-                [[obj editor] selectAsGroup];
-                [obj release];
-                [self setNeedsDisplay: YES];
+              obj = [obj initFromData: objdesc
+                               inView: self zoomFactor: zFactor];
+              [objects addObject: obj];
+              [[obj editor] selectAsGroup];
+              [obj release];
+              [self setNeedsDisplay: YES];
             }
         }
     }
+  else if ([imgType isEqualToString:NSTIFFPboardType])
+    {
+      NSImage *tmpNSImage;
+      GRImage *tmpGRImage;
+      NSMutableDictionary *properties;
+      NSString *str;
+      NSPoint pos;
+      NSSize size;
+    
+      tmpNSImage = [[NSImage alloc] initWithData:[pboard dataForType:NSTIFFPboardType]];
+    
+      size = [tmpNSImage size];
+      pos = NSMakePoint(50, pageRect.size.height-50);
+      properties = [[NSMutableDictionary alloc] init];
+      [properties autorelease];
+      str = [NSString stringWithFormat: @"%.3f", pos.x];
+      [properties setObject: str forKey: @"posx"];
+      str = [NSString stringWithFormat: @"%.3f", pos.y];
+      [properties setObject: str forKey: @"posy"];
+    
+      str = [NSString stringWithFormat: @"%.3f", size.width];
+      [properties setObject: str forKey: @"width"];
+      str = [NSString stringWithFormat: @"%.3f", -size.height];
+      [properties setObject: str forKey: @"height"];
+    
+      tmpGRImage = [[GRImage alloc] initInView: self zoomFactor:zFactor withProperties:properties];
+      [tmpGRImage setImage:tmpNSImage];
+      [objects addObject:tmpGRImage];
+      [tmpGRImage release];
+      [tmpNSImage release];
+      [self setNeedsDisplay: YES];
+    }
 }
 
 /* ----- Undo Methods ----- */
@@ -1630,56 +1833,139 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
     [self verifyModifiersOfEvent: theEvent];
 }
 
-- (BOOL)performKeyEquivalent:(NSEvent *)theEvent
+
+/** respond to key equivalents which are not bound to menu items */
+-(BOOL)performKeyEquivalent: (NSEvent*)theEvent
 {
-    NSString *commchar = [theEvent charactersIgnoringModifiers];
-
-    if([commchar isEqualToString: @"t"]) {
-        [self editSelectedText];
-        return YES;
-    }
-    if([commchar isEqualToString: @"d"]) {
-        return YES;
-    }
-
-    return NO;
-}
-
-- (void)keyDown:(NSEvent *)theEvent
-{
-  unsigned short keyCode;
+  NSString *keyStr;
+  unichar keyCh;
   NSRect vRect, hiddRect;
   NSPoint vPoint;
-  CGFloat hiddRx, hiddRy, hiddRw, hiddRh;
+  CGFloat hiddRx, hiddRy, hiddRw, hiddRh;  
+  
+#ifdef __APPLE__
+  /* Apple is definitively broken here and on all versions tested it returns for the arrow key
+     also a KeyUp event, which it should not, as the Event is specified to be keyDown */
+  if ([theEvent type] == NSKeyUp)
+    return [super performKeyEquivalent:theEvent];
+#endif
 
-    keyCode = [theEvent keyCode];
+  keyCh = 0x0;
+  keyStr = [theEvent characters];
+  if ([keyStr length] > 0)
+    keyCh = [keyStr characterAtIndex:0];
 
-    if(keyCode == NSDeleteFunctionKey)
+  if (keyCh == NSDeleteFunctionKey || keyCh == NSDeleteCharacter)
     {
-        [self deleteSelectedObjects];
-    } else if(keyCode == NSPageUpFunctionKey)
-    {
-        vRect = [self visibleRect];
-        vPoint = vRect.origin;
-        hiddRx = vPoint.x;
-        hiddRy = vPoint.y + vRect.size.height;
-        hiddRw = vRect.size.width;
-        hiddRh = vRect.size.height;
-        hiddRect = NSMakeRect(hiddRx, hiddRy, hiddRw, hiddRh);
-        [self scrollRectToVisible: hiddRect];
-    } else if(keyCode == NSPageDownFunctionKey)
-    {
-        vRect = [self visibleRect];
-        vPoint = vRect.origin;
-        hiddRx = vPoint.x;
-        hiddRy = vPoint.y - vRect.size.height;
-        hiddRw = vRect.size.width;
-        hiddRh = vRect.size.height;
-        hiddRect = NSMakeRect(hiddRx, hiddRy, hiddRw, hiddRh);
-        [self scrollRectToVisible: hiddRect];
+      [self delete:self];
+      return YES;
     }
+  else if(keyCh == NSPageUpFunctionKey)
+    {
+      vRect = [self visibleRect];
+      vPoint = vRect.origin;
+      hiddRx = vPoint.x;
+      hiddRy = vPoint.y + vRect.size.height;
+      hiddRw = vRect.size.width;
+      hiddRh = vRect.size.height;
+      hiddRect = NSMakeRect(hiddRx, hiddRy, hiddRw, hiddRh);
+      [self scrollRectToVisible: hiddRect];
+      return YES;
+    }
+  else if(keyCh == NSPageDownFunctionKey)
+    {
+      vRect = [self visibleRect];
+      vPoint = vRect.origin;
+      hiddRx = vPoint.x;
+      hiddRy = vPoint.y - vRect.size.height;
+      hiddRw = vRect.size.width;
+      hiddRh = vRect.size.height;
+      hiddRect = NSMakeRect(hiddRx, hiddRy, hiddRw, hiddRh);
+      [self scrollRectToVisible: hiddRect];
+      return YES;
+    }    
+  else if (keyCh == NSLeftArrowFunctionKey)
+    {
+      NSLog(@"arrow left");
+      return YES;
+    }
+  else if (keyCh == NSUpArrowFunctionKey)
+    {
+      NSLog(@"arrow up");
+      return YES;
+    }
+  else if (keyCh == NSRightArrowFunctionKey)
+    {
+      NSLog(@"right left");
+      return YES;
+    }
+  else if (keyCh == NSDownArrowFunctionKey)
+    {
+      NSLog(@"down left");
+      return YES;
+    }
+  else
+    NSLog(@"keyCh %x", keyCh);
+  return [super performKeyEquivalent:theEvent];
 }
 
+/* override the menu for special context menus
+
+   While editing a path, display the handle options
+ */
+- (NSMenu *) menuForEvent: (NSEvent*)theEvent
+{
+  if ([[NSApp delegate] currentToolType] == whitearrowtool)
+    {
+      NSUInteger i;
+      NSUInteger selectedPaths;
+      GRBezierPath *path;
+      
+      selectedPaths = 0;
+      path = nil;
+      for(i = 0; i < [objects count]; i++)
+        {
+          GRDrawableObject *obj;
+          obj = [objects objectAtIndex: i];
+          
+          if([[obj editor] isSelected] && [obj isKindOfClass:[GRBezierPath class]])
+            {
+              selectedPaths++;
+              path = (GRBezierPath *)obj;
+            }
+        }
+
+      if (selectedPaths == 1)
+        {
+          NSMenu *menu;
+          NSMenuItem *menuItem;
+          
+          
+          menu = [[NSMenu alloc] initWithTitle: NSLocalizedString(@"Handles", @"")];
+          
+          menuItem = [[NSMenuItem alloc] init];
+          [menuItem setTitle: NSLocalizedString(@"Symmetric", @"")];
+          [menuItem setTarget: self];      
+          [menuItem setAction: @selector(changePointsOfCurrentPathToSymmetric:)];     
+          [menu addItem: menuItem];
+          [menuItem release];
+          
+          menuItem = [[NSMenuItem alloc] init];
+          [menuItem setTitle: NSLocalizedString(@"Cusp", @"")];
+          [menu addItem: menuItem];
+          [menuItem setTarget: self];      
+          [menuItem setAction: @selector(changePointsOfCurrentPathToCusp:)];     
+          [menuItem release];
+          
+          [menu autorelease];
+          return menu;
+        }
+    }
+
+  return [super menuForEvent: theEvent]; 
+}
+
+
 - (BOOL)acceptsFirstResponder
 {
     return YES;
@@ -1696,25 +1982,27 @@ float zFactors[ZOOM_FACTORS] = {0.25, 0.33, 0.5, 0.66, 0.75, 1, 1.25, 1.5, 2, 2.
   NSRect       frameRect;
   NSBezierPath *bzp;
 
-    frameRect = [self frame];
+  frameRect = [self frame];
     
-    [[NSColor whiteColor] set];
-    NSRectFill(rect);
+  [[NSColor whiteColor] set];
+  NSRectFill(rect);
 
-    [[NSColor lightGrayColor] set];
-
-    bzp = [NSBezierPath bezierPath];
-    [bzp moveToPoint:NSMakePoint(0, zmdRect.origin.y)];
-    [bzp lineToPoint:NSMakePoint(frameRect.size.width, zmdRect.origin.y)];
-    [bzp lineToPoint:NSMakePoint(rect.size.width, zmdRect.origin.y)];
-    [bzp moveToPoint:NSMakePoint(0, zmdRect.origin.y + zmdRect.size.height)];
-    [bzp lineToPoint:NSMakePoint(frameRect.size.width, zmdRect.origin.y + zmdRect.size.height)];
-    [bzp moveToPoint:NSMakePoint(zmdRect.origin.x, 0)];
-    [bzp lineToPoint:NSMakePoint(zmdRect.origin.x, frameRect.size.height)];
-    [bzp moveToPoint:NSMakePoint(zmdRect.origin.x + zmdRect.size.width, 0)];
-    [bzp lineToPoint:NSMakePoint(zmdRect.origin.x + zmdRect.size.width, frameRect.size.height)];
-    [bzp stroke];
+  if ([[NSGraphicsContext currentContext] isDrawingToScreen])
+    {
+      [[NSColor lightGrayColor] set];
 
+      bzp = [NSBezierPath bezierPath];
+      [bzp moveToPoint:NSMakePoint(0, zmdRect.origin.y)];
+      [bzp lineToPoint:NSMakePoint(frameRect.size.width, zmdRect.origin.y)];
+      [bzp lineToPoint:NSMakePoint(rect.size.width, zmdRect.origin.y)];
+      [bzp moveToPoint:NSMakePoint(0, zmdRect.origin.y + zmdRect.size.height)];
+      [bzp lineToPoint:NSMakePoint(frameRect.size.width, zmdRect.origin.y + zmdRect.size.height)];
+      [bzp moveToPoint:NSMakePoint(zmdRect.origin.x, 0)];
+      [bzp lineToPoint:NSMakePoint(zmdRect.origin.x, frameRect.size.height)];
+      [bzp moveToPoint:NSMakePoint(zmdRect.origin.x + zmdRect.size.width, 0)];
+      [bzp lineToPoint:NSMakePoint(zmdRect.origin.x + zmdRect.size.width, frameRect.size.height)];
+      [bzp stroke];
+    }
 
     for(i = 0; i < [objects count]; i++)
         [(GRDrawableObject *)[objects objectAtIndex: i] draw];
