@@ -2,7 +2,7 @@
  Project: Graphos
  GRDrawableObject.m
 
- Copyright (C) 2008-2013 GNUstep Application Project
+ Copyright (C) 2008-2015 GNUstep Application Project
 
  Author: Ing. Riccardo Mottola
 
@@ -38,6 +38,11 @@
     [super dealloc];
 }
 
+- (GRObjectEditor *)allocEditor
+{
+  return [[GRObjectEditor alloc] initEditor:self];
+}
+
 /** initializes by using the properties array as defaults */
 - (id)initInView:(GRDocView *)aView
       zoomFactor:(CGFloat)zf
@@ -49,8 +54,6 @@
       NSColor *newColor;
       id val;
 
-      NSLog(@"initInView properties of GRDrawableObject");
-
       docView = aView;
       zmFactor = zf;
       stroked = YES;
@@ -81,6 +84,8 @@
       val = [properties objectForKey: @"locked"];
       if (val)
 	locked = [val boolValue];
+
+      editor = [self allocEditor];
     }
   return self;
 }
@@ -108,20 +113,27 @@
 
 - (id)copyWithZone:(NSZone *)zone
 {
-    GRDrawableObject *objCopy;
-    GRObjectEditor *editorCopy;
+  GRDrawableObject *objCopy;
+  GRObjectEditor *editorCopy;
 
-    objCopy = (GRDrawableObject *)NSCopyObject(self, 0, zone);
-    editorCopy = [[self editor] copy];
-    [editorCopy setObject: objCopy];
-    
-    objCopy->docView = [self view];
-    objCopy->editor = editorCopy;
+  objCopy = [[[self class] allocWithZone:zone] init];
 
-    objCopy->strokeColor = [[strokeColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
-    objCopy->fillColor = [[fillColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+  objCopy->visible = visible;
+  objCopy->locked = locked;
+  objCopy->zmFactor = zmFactor;
+  objCopy->stroked = stroked;
+  objCopy->filled = filled;
     
-    return objCopy;
+  editorCopy = [[self editor] copy];
+  [editorCopy setObject: objCopy];
+    
+  objCopy->docView = [self view];
+  objCopy->editor = editorCopy;
+
+  objCopy->strokeColor = [[strokeColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+  objCopy->fillColor = [[fillColor colorUsingColorSpaceName: NSCalibratedRGBColorSpace] retain];
+    
+  return objCopy;
 }
 
 - (BOOL)objectHitForSelection:(NSPoint)p
